<?php

namespace Modules\Qc\Http\Controllers;

use Exception;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Qc\Entities\DailyLabReportSbp;
use Modules\Qc\Entities\DailyLabReportSbpAttachment;

class DailyLabReportSbpController extends Controller
{
    public function uploadAttachment(Request $request)
    {
        try {
            $token = $request->get('attachment_token');
            $file = $request->file('file');
            $file_name = $file->getClientOriginalName();
            $filename = uniqid() . '_' . time() . '.' . $file_name;
            $file->move(public_path('dlr_sbp'), $filename);

            $dlr_sbp_attachment = new DailyLabReportSbpAttachment();
            $dlr_sbp_attachment->dlr_sbp_id = 0;
            $dlr_sbp_attachment->path = $filename;
            $dlr_sbp_attachment->token = $token;
            $dlr_sbp_attachment->save();

            $responseBody = $this->responseBody(true, "DailyLabReportSbpController", "uploaded", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "DailyLabReportSbpController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function allDailyLabReportSbps()
    {
        try {
            $query = "SELECT
            qc_daily_lab_report_sbp.*,
            qc_daily_lab_report_sbp_attachments.path,
            pp_customer_order_plans_data.manufacturing_order_no,
            st_products.product_name,
            hr_employee_sites.site
        FROM
            qc_daily_lab_report_sbp_attachments
            INNER JOIN
            qc_daily_lab_report_sbp
            ON
                qc_daily_lab_report_sbp_attachments.dlr_sbp_id = qc_daily_lab_report_sbp.dlr_sbp_id
            INNER JOIN
            pp_customer_order_plans_data
            ON
                qc_daily_lab_report_sbp.customer_order_plan_data_id = pp_customer_order_plans_data.customer_order_plan_data_id
            INNER JOIN
            st_products
            ON
                qc_daily_lab_report_sbp.product_primary_id = st_products.product_primary_id
            INNER JOIN
            hr_employee_sites
            ON
                qc_daily_lab_report_sbp.factory = hr_employee_sites.site_id
                GROUP BY
            qc_daily_lab_report_sbp.dlr_sbp_id
            ";

            $records = DB::select($query);
            $result = array();
            foreach ($records as $record) {
                array_push($result, [
                    "dlr_sbp_id" => $record->dlr_sbp_id,
                    "reference_no" => $record->reference_no,
                    "product_name" => $record->product_name,
                    "manufacturing_order_no" => $record->manufacturing_order_no,
                    "date" => $record->date,
                    "factory" => $record->site,
                    "description" => $record->description,
                    "path" => $record->path,
                ]);
            }
            $responseBody = $this->responseBody(true, "DailyLabReportSbpController", "allDailyLabReportSbps", $result);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "DailyLabReportSbpController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function saveDailyLabReportSbp(Request $request)
    {
        try {
            $sqr = new DailyLabReportSbp();
            $sqr->user_id = Auth::user()->id;
            $sqr->reference_no = $request->get('reference_no');
            $sqr->date = $request->get('date');
            $sqr->customer_order_plan_data_id = $request->get('customer_order_plan_data_id');
            $sqr->product_primary_id = $request->get('product_primary_id');
            $sqr->factory = $request->get('factory');
            $sqr->description = $request->get('description');
            if ($sqr->save()) {
                $token = $request->get('attachment_token');
                $attachment = DailyLabReportSbpAttachment::where('token', '=', $token)->first();
                if ($attachment) {
                    $attachment->dlr_sbp_id = $sqr->dlr_sbp_id;
                    $attachment->update();
                }
            }
            $responseBody = $this->responseBody(true, "DailyLabReportSbpController", "saveDailyLabReportSbp", $attachment);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "DailyLabReportSbpController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function update(Request $request, $id)
    {
        try {
            $sqr =  DailyLabReportSbp::find($id);
            $sqr->user_id = Auth::user()->id;
            $sqr->reference_no = $request->get('reference_no');
            $sqr->date = $request->get('date');
            $sqr->customer_order_plan_data_id = $request->get('customer_order_plan_data_id');
            $sqr->product_primary_id = $request->get('product_primary_id');
            $sqr->factory = $request->get('factory');
            $sqr->description = $request->get('description');
            if ($sqr->update()) {
                $token = $request->get('attachment_token');
                $attachments = DailyLabReportSbpAttachment::where('token', '=', $token)->get();
                if (count($attachments) > 1) {
                    if ($this->delete($id)) {
                        $attachment = DailyLabReportSbpAttachment::where('token', '=', $token)->first();
                        $attachment->dlr_sbp_id = (int)$id;
                        $attachment->update();
                    }
                }
                $responseBody = $this->responseBody(true, "DailyLabReportSbpController", "updated", count($attachments));
            }
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "DailyLabReportSbpController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }




    public function delete($id)
    {
        try {
            $attachment = DailyLabReportSbpAttachment::where('dlr_sbp_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            return true;
        } catch (Exception $exception) {
            return false;
        }
    }


    public function deleteRecord($id)
    {
        try {
            $record = DailyLabReportSbp::where('dlr_sbp_id', '=', $id)->get();
            foreach ($record as $data) {
                $data->delete();
            }
            $attachment = DailyLabReportSbpAttachment::where('dlr_sbp_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            $responseBody = $this->responseBody(true, "DailyLabReportSbpController", "deleted", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "DailyLabReportSbpController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function getDlrSbpData($id)
    {
        try {
            $query = "SELECT
            qc_daily_lab_report_sbp.*,
            qc_daily_lab_report_sbp_attachments.token
        FROM
            qc_daily_lab_report_sbp_attachments
            INNER JOIN
            qc_daily_lab_report_sbp
            ON
                qc_daily_lab_report_sbp_attachments.dlr_sbp_id = qc_daily_lab_report_sbp.dlr_sbp_id
        WHERE
            qc_daily_lab_report_sbp.dlr_sbp_id = {$id}";

            $sqr =  DB::select($query);

            $responseBody = $this->responseBody(true, "DailyLabReportSbpController", "getDlrSbpData", $sqr[0]);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "DailyLabReportSbpController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }



    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
