<?php

namespace Modules\Qc\Http\Controllers;

use Exception;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Qc\Entities\LocalOrderLtr;
use Modules\Qc\Entities\LocalOrderLtrAttachment;

class LocalOrderLtrController extends Controller
{
    public function uploadAttachment(Request $request)
    {
        try {
            $token = $request->get('attachment_token');
            $file = $request->file('file');
            $file_name = $file->getClientOriginalName();
            $filename = uniqid() . '_' . time() . '.' . $file_name;
            $file->move(public_path('local_order'), $filename);

            $local_order_attachment = new LocalOrderLtrAttachment();
            $local_order_attachment->local_order_id = 0;
            $local_order_attachment->path = $filename;
            $local_order_attachment->token = $token;
            $local_order_attachment->save();

            $responseBody = $this->responseBody(true, "LocalOrderLtrController", "uploaded", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "LocalOrderLtrController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function allRecords()
    {
        try {
            $query = "SELECT
            qc_local_order_ltrs.*,
            qc_local_order_ltr_attachments.path,
            pp_customer_order_plans_data.manufacturing_order_no,
            st_products.product_name,
            hr_employee_sites.site
        FROM
            qc_local_order_ltr_attachments
            INNER JOIN
            qc_local_order_ltrs
            ON
                qc_local_order_ltr_attachments.local_order_id = qc_local_order_ltrs.local_order_id
            INNER JOIN
            pp_customer_order_plans_data
            ON
                qc_local_order_ltrs.customer_order_plan_data_id = pp_customer_order_plans_data.customer_order_plan_data_id
            INNER JOIN
            st_products
            ON
                qc_local_order_ltrs.product_primary_id = st_products.product_primary_id
            INNER JOIN
            hr_employee_sites
            ON
                qc_local_order_ltrs.factory = hr_employee_sites.site_id
                GROUP BY
            qc_local_order_ltrs.local_order_id
            ";

            $records = DB::select($query);
            $result = array();
            foreach ($records as $record) {
                array_push($result, [
                    "local_order_id" => $record->local_order_id,
                    "reference_no" => $record->reference_no,
                    "product_name" => $record->product_name,
                    "manufacturing_order_no" => $record->manufacturing_order_no,
                    "date" => $record->date,
                    "factory" => $record->site,
                    "description" => $record->description,
                    "path" => $record->path,
                ]);
            }
            $responseBody = $this->responseBody(true, "LocalOrderLtrController", "allRecords", $result);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "LocalOrderLtrController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function saveRecord(Request $request)
    {
        try {
            $localOrder = new LocalOrderLtr();
            $localOrder->user_id = Auth::user()->id;
            $localOrder->reference_no = $request->get('reference_no');
            $localOrder->date = $request->get('date');
            $localOrder->customer_order_plan_data_id = $request->get('customer_order_plan_data_id');
            $localOrder->product_primary_id = $request->get('product_primary_id');
            $localOrder->factory = $request->get('factory');
            $localOrder->description = $request->get('description');
            if ($localOrder->save()) {
                $token = $request->get('attachment_token');
                $attachment = LocalOrderLtrAttachment::where('token', '=', $token)->first();
                if ($attachment) {
                    $attachment->local_order_id = $localOrder->local_order_id;
                    $attachment->update();
                }
            }
            $responseBody = $this->responseBody(true, "LocalOrderLtrController", "saveRecord", $attachment);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "LocalOrderLtrController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function update(Request $request, $id)
    {
        try {
            $localOrder = LocalOrderLtr::find($id);
            $localOrder->user_id = Auth::user()->id;
            $localOrder->reference_no = $request->get('reference_no');
            $localOrder->date = $request->get('date');
            $localOrder->customer_order_plan_data_id = $request->get('customer_order_plan_data_id');
            $localOrder->product_primary_id = $request->get('product_primary_id');
            $localOrder->factory = $request->get('factory');
            $localOrder->description = $request->get('description');
            if ($localOrder->update()) {
                $token = $request->get('attachment_token');
                $attachments = LocalOrderLtrAttachment::where('token', '=', $token)->get();
                if (count($attachments) > 1) {
                    if ($this->delete($id)) {
                        $attachment = LocalOrderLtrAttachment::where('token', '=', $token)->first();
                        $attachment->local_order_id = (int)$id;
                        $attachment->update();
                    }
                }
                $responseBody = $this->responseBody(true, "LocalOrderLtrController", "updated", count($attachments));
            }
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "LocalOrderLtrController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }




    public function delete($id)
    {
        try {
            $attachment = LocalOrderLtrAttachment::where('local_order_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            return true;
        } catch (Exception $exception) {
            return false;
        }
    }


    public function deleteRecord($id)
    {
        try {
            $record = LocalOrderLtr::where('local_order_id', '=', $id)->get();
            foreach ($record as $data) {
                $data->delete();
            }
            $attachment = LocalOrderLtrAttachment::where('local_order_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            $responseBody = $this->responseBody(true, "LocalOrderLtrController", "deleted", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "LocalOrderLtrController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function getData($id)
    {
        try {
            $query = "SELECT
            qc_local_order_ltrs.*,
            qc_local_order_ltr_attachments.token
        FROM
            qc_local_order_ltr_attachments
            INNER JOIN
            qc_local_order_ltrs
            ON
                qc_local_order_ltr_attachments.local_order_id = qc_local_order_ltrs.local_order_id
        WHERE
            qc_local_order_ltrs.local_order_id = {$id}";

            $localOrder =  DB::select($query);

            $responseBody = $this->responseBody(true, "LocalOrderLtrController", "getData", $localOrder[0]);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "LocalOrderLtrController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }



    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
